import 'package:flutter/cupertino.dart';
import 'package:hive/hive.dart';

mixin Restorable<T> {
  @protected
  Box<T> _box;
  @protected
  String get key;

  Future<T> getStored() async {
    try {
      registerAdapters();
      _box = await Hive.openBox(key);
      return _box.get(key);
    } catch (e) {
      await deleteState();
      return null;
    }
  }

  void registerAdapters();

  Future<void> save(T value) async {
    await _box?.put(key, value);
  }

  Future<void> deleteState() async {
    await Hive.deleteBoxFromDisk(key);
    _box = await Hive.openBox(key);
  }
}
